# Generated by Django 3.2.9 on 2021-12-11 15:56

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0023_auto_20211211_1852'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bloodresultslipid',
            name='chol_value',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=8, null=True, validators=[django.core.validators.MinValueValidator(0.0), django.core.validators.MaxValueValidator(999.0)], verbose_name='Total Cholesterol'),
        ),
        migrations.AlterField(
            model_name='historicalbloodresultslipid',
            name='chol_value',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=8, null=True, validators=[django.core.validators.MinValueValidator(0.0), django.core.validators.MaxValueValidator(999.0)], verbose_name='Total Cholesterol'),
        ),
    ]



# Generated by Django 3.0.9 on 2020-12-04 00:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0008_auto_20201204_0312"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], default="Yes", max_length=15
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], default="Yes", max_length=15
            ),
        ),
    ]



from .death_report import DeathReportFormValidator
from .end_of_study import EndOfStudyFormValidator
from .protocol_deviation_violation import ProtocolDeviationViolationFormValidator
from .study_day_form_validator_mixin import StudyDayFormValidatorMixin



from edc_visit_schedule import Crf, FormsCollection

crfs_prn = FormsCollection(
    Crf(show_order=212, model="mocca_subject.bloodresultslipid"),
    Crf(show_order=214, model="mocca_subject.cd4result"),
    Crf(show_order=215, model="mocca_subject.glucosefollowup"),
    Crf(show_order=216, model="mocca_subject.viralloadresult"),
    name="prn",
)

all_crfs = [
    Crf(show_order=110, model="mocca_subject.clinicalreview"),
    Crf(show_order=111, model="mocca_subject.indicators"),
    Crf(show_order=112, model="mocca_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="mocca_subject.dminitialreview", required=False),
    Crf(show_order=116, model="mocca_subject.htninitialreview", required=False),
    Crf(show_order=118, model="mocca_subject.cholinitialreview", required=False),
    Crf(show_order=120, model="mocca_subject.hivreview", required=False),
    Crf(show_order=130, model="mocca_subject.dmreview", required=False),
    Crf(show_order=140, model="mocca_subject.htnreview", required=False),
    Crf(show_order=142, model="mocca_subject.cholreview", required=False),
    Crf(show_order=145, model="mocca_subject.medications"),
    Crf(show_order=150, model="mocca_subject.drugrefillhtn", required=False),
    Crf(show_order=160, model="mocca_subject.drugrefilldm", required=False),
    Crf(show_order=170, model="mocca_subject.drugrefillhiv", required=False),
    Crf(show_order=175, model="mocca_subject.drugrefillchol", required=False),
    Crf(show_order=185, model="mocca_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="mocca_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="mocca_subject.htnmedicationadherence", required=False),
    Crf(show_order=197, model="mocca_subject.cholmedicationadherence", required=False),
    Crf(show_order=200, model="mocca_subject.complicationsfollowup", required=False),
    Crf(show_order=230, model="mocca_subject.nextappointment"),
]
crfs = FormsCollection(*all_crfs, name="all")

crfs_unscheduled = FormsCollection(
    *all_crfs,
    name="unscheduled",
)

crfs_missed = FormsCollection(
    Crf(show_order=10, model="mocca_subject.subjectvisitmissed"),
    name="missed",
)


crfs_d1 = FormsCollection(
    Crf(show_order=100, model="mocca_subject.clinicalreviewbaseline"),
    Crf(show_order=110, model="mocca_subject.indicators"),
    Crf(show_order=120, model="mocca_subject.hivinitialreview", required=False),
    Crf(show_order=130, model="mocca_subject.dminitialreview", required=False),
    Crf(show_order=140, model="mocca_subject.htninitialreview", required=False),
    Crf(show_order=142, model="mocca_subject.cholinitialreview", required=False),
    Crf(show_order=143, model="mocca_subject.medications"),
    Crf(show_order=145, model="mocca_subject.drugrefillhtn", required=False),
    Crf(show_order=150, model="mocca_subject.drugrefilldm", required=False),
    Crf(show_order=155, model="mocca_subject.drugrefillhiv", required=False),
    Crf(show_order=157, model="mocca_subject.drugrefillchol", required=False),
    Crf(show_order=160, model="mocca_subject.hivmedicationadherence", required=False),
    Crf(show_order=162, model="mocca_subject.dmmedicationadherence", required=False),
    Crf(show_order=164, model="mocca_subject.htnmedicationadherence", required=False),
    Crf(show_order=166, model="mocca_subject.cholmedicationadherence", required=False),
    Crf(show_order=168, model="mocca_subject.otherbaselinedata"),
    Crf(show_order=170, model="mocca_subject.complicationsbaseline"),
    Crf(show_order=171, model="mocca_subject.glucosebaseline"),
    Crf(show_order=212, model="mocca_subject.bloodresultslipid"),
    Crf(show_order=400, model="mocca_subject.nextappointment"),
    name="day1",
)



import csv
import os
import sys
from typing import List, Optional

from django.apps import apps as django_apps
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.management import color_style
from edc_constants.constants import FEMALE, MALE
from edc_model.stubs import BaseUuidModelStub
from tqdm import tqdm

from mocca_lists.models import MoccaOriginalSites

style = color_style()


class ImportMoccaError(Exception):
    pass


def birthyear_in_range(x: int) -> int:
    if not (1900 <= x <= 2003):
        raise TypeError(f"Birth year out of range. Got {x}")
    return x


def import_mocca_register(
    path: Optional[str] = None,
    verbose: bool = None,
    force_delete: Optional[bool] = None,
):
    model_cls = django_apps.get_model("mocca_screening.moccaregister")
    fieldnames = [
        "country",
        "site",
        "screening_id",
        "study_id",
        "initials",
        "birth_year",
        "age_in_years",
        "gender",
    ]
    path = os.path.expanduser(path or settings.MOCCA_REGISTER_FILE)
    try:
        if not os.path.exists(path):
            raise ImportMoccaError(path)
    except TypeError:
        raise ImportMoccaError(f"Invalid path. Got {path}.")
    if verbose:
        sys.stdout.write(
            f"\nImporting MOCCA register from '{path}' "
            f"into {model_cls._meta.label_lower}\n"
        )

    if model_cls.objects.all().count() > 0:
        if force_delete:
            model_cls.objects.all().delete()
        else:
            raise ImportMoccaError(
                "Unable to continue. Not deleting existing records. "
                "Use `force_delete` to override"
            )

    import_file(path, model_cls, fieldnames)

    if verbose:
        sys.stdout.write("Done.\n")


def import_file(
    path: str,
    model_cls: BaseUuidModelStub,
    fieldnames: List[str],
) -> None:
    sys.stdout.write(style.MIGRATE_HEADING("\n Importing mocca register.\n"))
    with open(path, "r") as f:
        reader = csv.DictReader(f, fieldnames=fieldnames)
        total = len([row for row in reader])
    with open(path, "r") as f:
        reader = csv.DictReader(f, fieldnames=fieldnames)
        for index, row in tqdm(enumerate(reader), total=total):
            if index == 0:
                continue
            try:
                model_cls.objects.get(mocca_study_identifier=row["study_id"])
            except ObjectDoesNotExist:
                mocca_site = MoccaOriginalSites.objects.get(name=row["site"].lower())
                model_cls.objects.create(
                    mocca_country=row["country"].lower(),
                    mocca_site=mocca_site,
                    mocca_screening_identifier=row["screening_id"],
                    mocca_study_identifier=row["study_id"],
                    initials=(row["initials"] or "").upper(),
                    birth_year=birthyear_in_range(int(row["birth_year"])),
                    age_in_years=row["age_in_years"],
                    gender=MALE if row["gender"].lower() == "male" else FEMALE,
                )



from mocca_lists.models import ArvRegimens


class Rx(ArvRegimens):
    """Declared as a proxy model for the autocomplete.

    turns out this was not necessary.
    """

    class Meta:
        proxy = True
        verbose_name = "ARV Regimens"
        verbose_name_plural = "ARV Regimens"



# Generated by Django 3.0.9 on 2020-12-04 00:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0011_auto_20201204_0347"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="date_last_called",
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="date_last_called",
            field=models.DateField(null=True),
        ),
    ]



# Generated by Django 3.2.9 on 2022-01-09 14:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0027_auto_20220109_1717'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='glucose',
            name='assay_datetime',
        ),
        migrations.RemoveField(
            model_name='glucose',
            name='requisition',
        ),
        migrations.RemoveField(
            model_name='historicalglucose',
            name='assay_datetime',
        ),
        migrations.RemoveField(
            model_name='historicalglucose',
            name='requisition',
        ),
        migrations.RemoveField(
            model_name='historicalglucosebaseline',
            name='assay_datetime',
        ),
        migrations.RemoveField(
            model_name='historicalglucosebaseline',
            name='requisition',
        ),
    ]



from dateutil.relativedelta import relativedelta
from django import forms
from django.test import TestCase
from edc_constants.constants import FEMALE, MOBILE_NUMBER, NO, YES
from edc_utils import get_utcnow
from pytz import timezone

from mocca_consent.forms import SubjectConsentForm, SubjectConsentFormValidator

from ..mocca_test_case_mixin import MoccaTestCaseMixin


def get_now():
    return get_utcnow().astimezone(timezone("Africa/Kampala"))


class TestSubjectConsent(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.eligibility_datetime = get_utcnow() - relativedelta(days=2)  # yesterday

    def test_form_validator_ok(self):
        subject_screening = self.get_subject_screening(
            eligibility_datetime=self.eligibility_datetime,
            gender=FEMALE,
        )
        consent_datetime = get_now()
        cleaned_data = dict(
            screening_identifier=subject_screening.screening_identifier,
            dob=subject_screening.report_datetime.date()
            - relativedelta(years=subject_screening.age_in_years),
            consent_datetime=consent_datetime,
            identity_type=MOBILE_NUMBER,
            identity="77777777",
            confirm_identity="77777777",
            gender=FEMALE,
        )
        validator = SubjectConsentFormValidator(
            cleaned_data=cleaned_data,
        )
        validator.clean()

    def test_form_validator_consent_before_eligibility_datetime(self):
        subject_screening = self.get_subject_screening(
            eligibility_datetime=self.eligibility_datetime,
            gender=FEMALE,
        )
        consent_datetime = subject_screening.eligibility_datetime - relativedelta(minutes=10)
        consent_datetime = consent_datetime.astimezone(timezone("Africa/Kampala"))
        cleaned_data = dict(
            screening_identifier=subject_screening.screening_identifier,
            dob=subject_screening.report_datetime.date()
            - relativedelta(years=subject_screening.age_in_years),
            consent_datetime=consent_datetime,
            identity_type=MOBILE_NUMBER,
            identity="77777777",
            confirm_identity="77777777",
            gender=FEMALE,
        )
        validator = SubjectConsentFormValidator(
            cleaned_data=cleaned_data,
        )
        self.assertRaises(forms.ValidationError, validator.clean)
        with self.assertRaises(forms.ValidationError) as cm:
            validator.clean()
        self.assertIn("consent_datetime", str(cm.exception))

    def test_form_validator_consent_after_eligibility_datetime(self):
        subject_screening = self.get_subject_screening(
            eligibility_datetime=self.eligibility_datetime,
            gender=FEMALE,
        )
        consent_datetime = subject_screening.eligibility_datetime + relativedelta(minutes=1)
        consent_datetime = consent_datetime.astimezone(timezone("Africa/Kampala"))
        cleaned_data = dict(
            screening_identifier=subject_screening.screening_identifier,
            dob=subject_screening.report_datetime.date()
            - relativedelta(years=subject_screening.age_in_years),
            consent_datetime=consent_datetime,
            identity_type=MOBILE_NUMBER,
            identity="77777777",
            confirm_identity="77777777",
            gender=FEMALE,
        )
        validator = SubjectConsentFormValidator(
            cleaned_data=cleaned_data,
        )
        try:
            validator.clean()
        except forms.ValidationError:
            self.fail("ValidationError unexpectedly raised")

    def test_model_consent(self):
        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        self.assertIsNotNone(subject_consent.subject_identifier)

        subject_screening = self.get_subject_screening()
        subject_consent = self.get_subject_consent(subject_screening)
        self.assertIsNotNone(subject_consent.subject_identifier)

    def test_subjectconsent_form(self):

        subject_screening = self.get_subject_screening()
        data = dict(
            screening_identifier=subject_screening.screening_identifier,
            dob=subject_screening.report_datetime.date()
            - relativedelta(years=subject_screening.age_in_years),
            consent_datetime=get_now(),
            identity_type=MOBILE_NUMBER,
            identity="77777777",
            confirm_identity="77777777",
            gender=FEMALE,
            assessment_score=YES,
            consent_copy=YES,
            consent_reviewed=YES,
            consent_signature=YES,
            first_name="XAVIER",
            initials="XX",
            is_dob_estimated="-",
            is_incarcerated=NO,
            is_literate=YES,
            last_name="XAVIER",
            study_questions=YES,
            site=subject_screening.site,
            language="en",
            user_created="erikvw",
        )
        form = SubjectConsentForm(data=data, instance=None)
        form.is_valid()
        self.assertEqual(form._errors, {})
        form.save()



# Generated by Django 3.0.9 on 2021-01-15 01:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_subject", "0005_auto_20210114_2257"),
    ]

    operations = [
        migrations.AddField(
            model_name="dminitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                ],
                default="mocca_clinic",
                max_length=25,
                verbose_name="Was the diagnosis date estimated?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicaldminitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                ],
                default="mocca_clinic",
                max_length=25,
                verbose_name="Was the diagnosis date estimated?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhivinitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                ],
                default="mocca_clinic",
                max_length=25,
                verbose_name="Was the diagnosis date estimated?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalhtninitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                ],
                default="mocca_clinic",
                max_length=25,
                verbose_name="Was the diagnosis date estimated?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="hivinitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                ],
                default="mocca_clinic",
                max_length=25,
                verbose_name="Was the diagnosis date estimated?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="htninitialreview",
            name="dx_location",
            field=models.CharField(
                choices=[
                    ("hospital", "Hospital"),
                    ("gov_clinic", "Government clinic"),
                    ("private_clinic", "Private clinic"),
                    ("private_doctor", "Private doctor"),
                    ("mocca_clinic", "MOCCA study clinic"),
                ],
                default="mocca_clinic",
                max_length=25,
                verbose_name="Was the diagnosis date estimated?",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="reason",
            field=models.CharField(
                choices=[
                    ("scheduled", "Scheduled visit (study)"),
                    ("unscheduled", "Routine or unscheduled visit"),
                    ("missed", "Missed visit"),
                ],
                help_text="Only baseline (0m), 6m and 12m are considered `scheduled` visits as per the MOCCA protocol.If `missed' complete CRF Missed Visit Report.",
                max_length=25,
                verbose_name="What is the reason for this visit report?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="reason_unscheduled",
            field=models.CharField(
                choices=[
                    ("routine_non_study", "Routine scheduled (non-study)"),
                    ("patient_unwell_outpatient", "Patient unwell / unscheduled"),
                    ("drug_refill", "Drug refill only"),
                    ("OTHER", "Other"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If 'unscheduled', provide reason for the unscheduled visit",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="reason",
            field=models.CharField(
                choices=[
                    ("scheduled", "Scheduled visit (study)"),
                    ("unscheduled", "Routine or unscheduled visit"),
                    ("missed", "Missed visit"),
                ],
                help_text="Only baseline (0m), 6m and 12m are considered `scheduled` visits as per the MOCCA protocol.If `missed' complete CRF Missed Visit Report.",
                max_length=25,
                verbose_name="What is the reason for this visit report?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="reason_unscheduled",
            field=models.CharField(
                choices=[
                    ("routine_non_study", "Routine scheduled (non-study)"),
                    ("patient_unwell_outpatient", "Patient unwell / unscheduled"),
                    ("drug_refill", "Drug refill only"),
                    ("OTHER", "Other"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If 'unscheduled', provide reason for the unscheduled visit",
            ),
        ),
    ]



# Generated by Django 3.0.9 on 2020-12-15 03:02

import uuid

import _socket
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_model.models.fields.other_charfield
import edc_sites.models
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0002_alter_domain_unique"),
        ("mocca_screening", "0003_carestatus"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalsubjectrefusal",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Refusal to Consent",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectrefusal",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Refusal to Consent",
                "verbose_name_plural": "Refusal to Consent",
            },
        ),
        migrations.AddField(
            model_name="carestatus",
            name="report_datetime",
            field=models.DateTimeField(
                default=edc_utils.date.get_utcnow,
                help_text="Date and time of report.",
                verbose_name="Report Date and Time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="pregnant",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not Applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="requires_acute_care",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="Does the patient require acute care including in-patient admission",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="willing_to_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If No, complete refusal form.",
                max_length=25,
                verbose_name="Has the patient expressed willingness to participate in the `MOCCA extension` trial",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="pregnant",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not Applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Is the patient pregnant?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="requires_acute_care",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=25,
                verbose_name="Does the patient require acute care including in-patient admission",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="willing_to_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If No, complete refusal form.",
                max_length=25,
                verbose_name="Has the patient expressed willingness to participate in the `MOCCA extension` trial",
            ),
        ),
        migrations.CreateModel(
            name="SubjectRefusalScreening",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "reason",
                    models.CharField(
                        choices=[
                            ("unwilling_to_say", "I am unwilling to say"),
                            ("dont_have_time", "I don't have time"),
                            ("stigma", "I am worried about stigma"),
                            ("must_consult_spouse", "I need to consult my spouse"),
                            (
                                "dont_want_medication",
                                "I don't want to take any more medication",
                            ),
                            ("dont_want_to_join", "I don't want to take part"),
                            (
                                "need_to_think_about_it",
                                "I haven't had a chance to think about it",
                            ),
                            ("moving", "I am moving to another area"),
                            ("OTHER", "Other, please specify"),
                        ],
                        max_length=25,
                        verbose_name="Reason for refusal to screen",
                    ),
                ),
                (
                    "other_reason",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "mocca_register",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_screening.MoccaRegister",
                        verbose_name="MOCCA (original) register details",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Refusal to Screen",
                "verbose_name_plural": "Refusal to Screen",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalSubjectRefusalScreening",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "reason",
                    models.CharField(
                        choices=[
                            ("unwilling_to_say", "I am unwilling to say"),
                            ("dont_have_time", "I don't have time"),
                            ("stigma", "I am worried about stigma"),
                            ("must_consult_spouse", "I need to consult my spouse"),
                            (
                                "dont_want_medication",
                                "I don't want to take any more medication",
                            ),
                            ("dont_want_to_join", "I don't want to take part"),
                            (
                                "need_to_think_about_it",
                                "I haven't had a chance to think about it",
                            ),
                            ("moving", "I am moving to another area"),
                            ("OTHER", "Other, please specify"),
                        ],
                        max_length=25,
                        verbose_name="Reason for refusal to screen",
                    ),
                ),
                (
                    "other_reason",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "mocca_register",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_screening.MoccaRegister",
                        verbose_name="MOCCA (original) register details",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Refusal to Screen",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from django.conf import settings
from django.db import models
from django.utils.safestring import mark_safe
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_dx_review.model_mixins import ClinicalReviewModelMixin
from edc_model import models as edc_models

from mocca_lists.models import ReasonsForTesting

from ..model_mixins import CrfModelMixin


class ClinicalReview(
    ClinicalReviewModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):

    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="+",
    )

    hiv_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for HIV infection?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=mark_safe(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )

    hiv_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    hiv_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="hiv_test_reason",
        verbose_name="Why was the patient tested for HIV infection?",
        blank=True,
    )

    hiv_reason_other = edc_models.OtherCharField()

    hiv_dx = models.CharField(
        verbose_name=mark_safe(
            "As of today, was the patient <u>newly</u> diagnosed with HIV infection?"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    htn_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for hypertension?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=mark_safe(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )

    htn_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    htn_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="htn_test_reason",
        verbose_name="Why was the patient tested for hypertension?",
        blank=True,
    )

    htn_reason_other = edc_models.OtherCharField()

    htn_dx = models.CharField(
        verbose_name=mark_safe(
            "As of today, was the patient <u>newly</u> diagnosed with hypertension?"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    dm_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for diabetes?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=mark_safe(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )

    dm_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    dm_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="dm_reason",
        verbose_name="Why was the patient tested for diabetes?",
        blank=True,
    )

    dm_reason_other = edc_models.OtherCharField()

    dm_dx = models.CharField(
        verbose_name=mark_safe(
            "As of today, was the patient <u>newly</u> diagnosed with diabetes?"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    chol_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for high cholesterol?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=mark_safe(
            "Note: Select `not applicable` if diagnosis previously reported. <BR>"
            "`Since last seen` includes today.<BR>"
            "If `yes', complete the initial review CRF<BR>"
            "If `not applicable`, complete the review CRF."
        ),
    )
    chol_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    chol_reason = models.ManyToManyField(
        ReasonsForTesting,
        related_name="chol_reason",
        verbose_name="Why was the patient tested for cholesterol?",
        blank=True,
    )

    chol_reason_other = edc_models.OtherCharField()

    chol_dx = models.CharField(
        verbose_name=mark_safe(
            "As of today, was the patient <u>newly</u> diagnosed with high cholesterol?"
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    complications = models.CharField(
        verbose_name="Since last seen, has the patient had any complications",
        max_length=15,
        choices=YES_NO,
        help_text="If Yes, complete the `Complications` CRF",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Clinical Review"
        verbose_name_plural = "Clinical Reviews"



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.safestring import mark_safe
from edc_constants.choices import YES_NO, YES_NO_NOT_REQUIRED
from edc_constants.constants import NOT_REQUIRED, YES
from edc_model import models as edc_models
from edc_model.models import BaseUuidModel
from edc_vitals import models as edc_vitals_models

from ..model_mixins import CrfModelMixin


class Indicators(CrfModelMixin, BaseUuidModel):
    weight = edc_vitals_models.WeightField(
        validators=[MinValueValidator(25), MaxValueValidator(200)],
        null=True,
        blank=True,
    )

    height = edc_vitals_models.HeightField(
        null=True,
        blank=True,
    )

    r1_taken = models.CharField(
        verbose_name=mark_safe("Was a blood pressure reading taken"),
        max_length=15,
        choices=YES_NO,
        default=YES,
    )

    r1_reason_not_taken = models.TextField(
        verbose_name="reason not taken", max_length=250, null=True, blank=True
    )

    sys_blood_pressure_r1 = edc_vitals_models.SystolicPressureField(null=True, blank=True)

    dia_blood_pressure_r1 = edc_vitals_models.DiastolicPressureField(null=True, blank=True)

    r2_taken = models.CharField(
        verbose_name=mark_safe("Was a <u>second</u> blood pressure reading taken"),
        max_length=15,
        choices=YES_NO_NOT_REQUIRED,
        default=NOT_REQUIRED,
    )

    r2_reason_not_taken = models.TextField(max_length=250, null=True, blank=True)

    sys_blood_pressure_r2 = edc_vitals_models.SystolicPressureField(null=True, blank=True)

    dia_blood_pressure_r2 = edc_vitals_models.DiastolicPressureField(null=True, blank=True)

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Indicators"
        verbose_name_plural = "Indicators"



# Generated by Django 3.0.9 on 2020-12-04 16:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0013_auto_20201204_0613"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="moccaregistercontact",
            options={
                "ordering": ["report_datetime"],
                "verbose_name": "MOCCA Patient Register Contact",
                "verbose_name_plural": "MOCCA Patient Register Contacts",
            },
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Call?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Call?",
            ),
        ),
    ]



from multisite import SiteID

from .defaults import *  # noqa

print(f"Settings file {__file__}")  # noqa

SITE_ID = SiteID(default=1)
EDC_SITES_UAT_DOMAIN = True
ALLOWED_HOSTS = [
    "amana.uat.tz.mocca.clinicedc.org",
    "bunju.uat.tz.mocca.clinicedc.org",
    "hindu-mandal.uat.tz.mocca.clinicedc.org",
    "kisugu.uat.ug.mocca.clinicedc.org",
    "kiswa.uat.ug.mocca.clinicedc.org",
    "mkuranga.uat.tz.mocca.clinicedc.org",
    "mulago.uat.ug.mocca.clinicedc.org",
    "mwananyamala.uat.tz.mocca.clinicedc.org",
    "ndejje.uat.ug.mocca.clinicedc.org",
    "wakiso.uat.ug.mocca.clinicedc.org",
    "localhost",
]



# Generated by Django 3.2.10 on 2021-12-09 17:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_screening', '0014_auto_20210413_2000'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalsubjectscreening',
            name='real_eligibility_datetime',
            field=models.DateTimeField(editable=False, help_text='Date and time eligibility was determined relative to now', null=True),
        ),
        migrations.AddField(
            model_name='subjectscreening',
            name='real_eligibility_datetime',
            field=models.DateTimeField(editable=False, help_text='Date and time eligibility was determined relative to now', null=True),
        ),
        migrations.AlterField(
            model_name='carestatus',
            name='icc_not_in_reason',
            field=models.CharField(choices=[('icc_not_available', 'ICC not available (or closed) in this facility'), ('moved', 'Moved out of area'), ('dont_want', 'Personally chose not to continue with integrated care'), ('advised_to_vertical', 'Healthcare staff asked patient to return to vertical care'), ('referred_out', 'Referred to another facility without an ICC'), ('receives_community_care', 'Patient is currently receiving their HIV care in the community'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='If not receiving integrated care, why not?'),
        ),
        migrations.AlterField(
            model_name='historicalcarestatus',
            name='icc_not_in_reason',
            field=models.CharField(choices=[('icc_not_available', 'ICC not available (or closed) in this facility'), ('moved', 'Moved out of area'), ('dont_want', 'Personally chose not to continue with integrated care'), ('advised_to_vertical', 'Healthcare staff asked patient to return to vertical care'), ('referred_out', 'Referred to another facility without an ICC'), ('receives_community_care', 'Patient is currently receiving their HIV care in the community'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='If not receiving integrated care, why not?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='eligibility_datetime',
            field=models.DateTimeField(editable=False, help_text='Date and time eligibility was determined relative to report_datetime', null=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='icc_not_in_reason',
            field=models.CharField(choices=[('icc_not_available', 'ICC not available (or closed) in this facility'), ('moved', 'Moved out of area'), ('dont_want', 'Personally chose not to continue with integrated care'), ('advised_to_vertical', 'Healthcare staff asked patient to return to vertical care'), ('referred_out', 'Referred to another facility without an ICC'), ('receives_community_care', 'Patient is currently receiving their HIV care in the community'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='If not receiving integrated care, why not?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='screening_consent',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, verbose_name='Has the subject given his/her verbal consent to be screened for the MOCCA Extension study?'),
        ),
        migrations.AlterField(
            model_name='historicalsubjectscreening',
            name='willing_to_consent',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='If No, complete refusal form.', max_length=25, verbose_name='Has the patient expressed willingness to participate in the `MOCCA extension` study'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='eligibility_datetime',
            field=models.DateTimeField(editable=False, help_text='Date and time eligibility was determined relative to report_datetime', null=True),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='icc_not_in_reason',
            field=models.CharField(choices=[('icc_not_available', 'ICC not available (or closed) in this facility'), ('moved', 'Moved out of area'), ('dont_want', 'Personally chose not to continue with integrated care'), ('advised_to_vertical', 'Healthcare staff asked patient to return to vertical care'), ('referred_out', 'Referred to another facility without an ICC'), ('receives_community_care', 'Patient is currently receiving their HIV care in the community'), ('N/A', 'Not applicable')], default='N/A', max_length=25, verbose_name='If not receiving integrated care, why not?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='screening_consent',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No')], max_length=15, verbose_name='Has the subject given his/her verbal consent to be screened for the MOCCA Extension study?'),
        ),
        migrations.AlterField(
            model_name='subjectscreening',
            name='willing_to_consent',
            field=models.CharField(choices=[('Yes', 'Yes'), ('No', 'No'), ('N/A', 'Not applicable')], default='N/A', help_text='If No, complete refusal form.', max_length=25, verbose_name='Has the patient expressed willingness to participate in the `MOCCA extension` study'),
        ),
    ]



from .end_of_study import EndOfStudy
from .loss_to_followup import LossToFollowup
from .offschedule import OffSchedule
from .onschedule import OnSchedule
from .protocol_deviation_violation import ProtocolDeviationViolation



from django import forms
from edc_action_item.forms.action_item_form_mixin import ActionItemFormMixin
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import (
    raise_if_both_ago_and_actual_date,
    raise_if_clinical_review_does_not_exist,
)
from edc_form_validators.form_validator import FormValidator
from edc_model.utils import estimated_date_from_ago

from ..constants import DRUGS
from ..models import HtnInitialReview


class HtnInitialReviewFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        raise_if_both_ago_and_actual_date(cleaned_data=self.cleaned_data)
        self.required_if(DRUGS, field="managed_by", field_required="med_start_ago")
        if self.cleaned_data.get("med_start_ago") and self.cleaned_data.get("dx_ago"):
            if estimated_date_from_ago(
                data=self.cleaned_data, ago_field="med_start_ago"
            ) < estimated_date_from_ago(data=self.cleaned_data, ago_field="dx_ago"):
                raise forms.ValidationError(
                    {"med_start_ago": "Invalid. Cannot be before diagnosis."}
                )


class HtnInitialReviewForm(
    CrfModelFormMixin,
    ActionItemFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HtnInitialReviewFormValidator

    class Meta:
        model = HtnInitialReview
        fields = "__all__"



# Generated by Django 3.0.9 on 2020-12-05 15:34

import django_crypto_fields.fields.encrypted_text_field
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0018_auto_20201205_0424"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="next_appt_date",
            field=models.DateField(
                blank=True, help_text="auto-updated", null=True, verbose_name="Appt"
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="next_appt_date",
            field=models.DateField(
                blank=True, help_text="auto-updated", null=True, verbose_name="Appt"
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="contact_attempts",
            field=models.IntegerField(default=0, help_text="auto-updated"),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="date_last_called",
            field=models.DateField(help_text="auto-updated", null=True),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="notes",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="General notes",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="comment",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="Note",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="next_appt_date",
            field=models.DateField(blank=True, null=True, verbose_name="Next Appt."),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="contact_attempts",
            field=models.IntegerField(default=0, help_text="auto-updated"),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="date_last_called",
            field=models.DateField(help_text="auto-updated", null=True),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="notes",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="General notes",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="comment",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="Note",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="next_appt_date",
            field=models.DateField(blank=True, null=True, verbose_name="Next Appt."),
        ),
    ]



from django import forms
from edc_adherence.form_validator_mixin import MedicationAdherenceFormValidatorMixin
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_model.widgets import SliderWidget

from ..models import DmMedicationAdherence


class DmMedicationAdherenceFormValidator(
    MedicationAdherenceFormValidatorMixin,
    CrfFormValidatorMixin,
    FormValidator,
):
    pass


class DmMedicationAdherenceForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DmMedicationAdherenceFormValidator

    visual_score_slider = forms.CharField(
        label="Visual Score", widget=SliderWidget(attrs={"min": 0, "max": 100})
    )

    class Meta:
        model = DmMedicationAdherence
        fields = "__all__"



from django.test import TestCase
from edc_list_data.site_list_data import site_list_data


class TestList(TestCase):
    def test_(self):
        # TODO: Validate
        site_list_data.initialize()
        site_list_data.autodiscover()



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import ViralLoadResultForm
from ..models import ViralLoadResult
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(ViralLoadResult, site=mocca_subject_admin)
class ViralLoadResultAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = ViralLoadResultForm

    fieldsets = (
        (
            None,
            {"fields": ("subject_visit", "report_datetime")},
        ),
        ("Result", {"fields": ("drawn_date", "result", "quantifier")}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "quantifier": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }



model_data = {
    "edc_lab.consignee": [
        {
            "name": "The MOCCA Trial",
            "contact_name": "Josphine Birungi",
            "address": "LSTM Uganda",
            "postal_code": "-",
            "city": "Entebbe",
            "state": None,
            "country": "Uganda",
            "telephone": "555-5555",
            "mobile": "555-5555",
            "fax": None,
            "email": "josephine.birungi@mrcuganda.org",
        }
    ]
}

unique_field_data = {"edc_lab.consignee": {"name": ("-", "-")}}



from django.conf import settings
from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin
from .model_mixins import ReviewModelMixin


class HivReview(ReviewModelMixin, CrfModelMixin, edc_models.BaseUuidModel):

    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="+",
    )

    test_date = models.DateField(
        verbose_name="Date tested for HIV",
        null=True,
        blank=True,
        help_text="QUESTION_RETIRED",
    )

    dx = models.CharField(
        verbose_name="Has the patient been infected with HIV?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    arv_initiated = models.CharField(
        verbose_name="Has the patient started antiretroviral therapy (ART)?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="Select `not applicable` if previously reported.",
    )
    arv_initiation_actual_date = models.DateField(
        verbose_name="Date started antiretroviral therapy (ART)",
        null=True,
        blank=True,
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "HIV Review"
        verbose_name_plural = "HIV Review"



# Generated by Django 3.0.9 on 2020-12-04 20:47

import django_crypto_fields.fields.encrypted_char_field
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0015_auto_20201204_2339"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="comment",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Note",
            ),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="comment",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Note",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="best_tel",
            field=models.CharField(
                blank=True,
                choices=[
                    ("tel_1", "Tel/Mobile(1)"),
                    ("tel_2", "Tel/Mobile(2)"),
                    ("tel_3", "Tel/Mobile(3)"),
                ],
                help_text="If any, select the best telephone/mobile from above",
                max_length=15,
                null=True,
                verbose_name="Prefered Telephone / Mobile",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="best_tel",
            field=models.CharField(
                blank=True,
                choices=[
                    ("tel_1", "Tel/Mobile(1)"),
                    ("tel_2", "Tel/Mobile(2)"),
                    ("tel_3", "Tel/Mobile(3)"),
                ],
                help_text="If any, select the best telephone/mobile from above",
                max_length=15,
                null=True,
                verbose_name="Prefered Telephone / Mobile",
            ),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.html import format_html
from edc_model import models as edc_models

from mocca_lists.models import NonAdherenceReasons

from ..choices import MISSED_PILLS


class MedicationAdherenceModelMixin(models.Model):

    condition_label = "condition_label"

    visual_score_slider = models.CharField(
        verbose_name=format_html(
            f"Visual adherence score for <U>{condition_label}</U> medication"
        ),
        max_length=3,
        help_text="%",
    )

    visual_score_confirmed = models.IntegerField(
        verbose_name=format_html(
            "<B><font color='orange'>Interviewer</font></B>: "
            "please confirm the score indicated from above."
        ),
        validators=[MinValueValidator(0), MaxValueValidator(100)],
        help_text="%",
    )

    last_missed_pill = models.CharField(
        verbose_name=format_html(
            "When was the last time you missed taking your "
            f"<U>{condition_label}</U> medication?"
        ),
        max_length=25,
        choices=MISSED_PILLS,
    )

    missed_pill_reason = models.ManyToManyField(
        NonAdherenceReasons,
        verbose_name="Reasons for miss taking medication",
        blank=True,
    )

    other_missed_pill_reason = edc_models.OtherCharField()

    class Meta:
        abstract = True



# Generated by Django 3.2.10 on 2021-12-09 17:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_prn", "0002_auto_20210225_0604"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalunblindingrequest",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingrequest",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingrequest",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingrequest",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingrequest",
            name="requestor",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingrequest",
            name="site",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingreview",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingreview",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingreview",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingreview",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingreview",
            name="reviewer",
        ),
        migrations.RemoveField(
            model_name="historicalunblindingreview",
            name="site",
        ),
        migrations.RemoveField(
            model_name="unblindingrequest",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="unblindingrequest",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="unblindingrequest",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="unblindingrequest",
            name="requestor",
        ),
        migrations.RemoveField(
            model_name="unblindingrequest",
            name="site",
        ),
        migrations.RemoveField(
            model_name="unblindingreview",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="unblindingreview",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="unblindingreview",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="unblindingreview",
            name="reviewer",
        ),
        migrations.RemoveField(
            model_name="unblindingreview",
            name="site",
        ),
        migrations.DeleteModel(
            name="UnblindingRequestorUser",
        ),
        migrations.DeleteModel(
            name="UnblindingReviewerUser",
        ),
        migrations.RenameField(
            model_name="historicallosstofollowup",
            old_name="loss_category_other",
            new_name="ltfu_category_other",
        ),
        migrations.RenameField(
            model_name="losstofollowup",
            old_name="loss_category_other",
            new_name="ltfu_category_other",
        ),
        migrations.RenameField(
            model_name="historicallosstofollowup",
            old_name="loss_category",
            new_name="ltfu_category",
        ),
        migrations.RenameField(
            model_name="losstofollowup",
            old_name="loss_category",
            new_name="ltfu_category",
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="historicalprotocoldeviationviolation",
            name="tracking_identifier",
            field=models.CharField(db_index=True, max_length=32),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="tracking_identifier",
            field=models.CharField(max_length=32, unique=True),
        ),
        migrations.DeleteModel(
            name="HistoricalUnblindingRequest",
        ),
        migrations.DeleteModel(
            name="HistoricalUnblindingReview",
        ),
        migrations.DeleteModel(
            name="UnblindingRequest",
        ),
        migrations.DeleteModel(
            name="UnblindingReview",
        ),
    ]



from edc_constants.constants import (
    DEAD,
    DIABETES,
    FREE_OF_CHARGE,
    HIV,
    HOSPITALIZED,
    HYPERTENSION,
    NOT_APPLICABLE,
    OTHER,
    REFILL,
    ROUTINE_VISIT,
    STUDY_DEFINED_TIMEPOINT,
    UNKNOWN,
    UNWELL_VISIT,
)
from edc_ltfu.constants import LOST_TO_FOLLOWUP
from edc_offstudy.constants import LATE_EXCLUSION, WITHDRAWAL
from edc_transfer.constants import TRANSFERRED

list_data = {
    "mocca_lists.diagnosislocations": [
        ("hospital", "Hospital"),
        ("gov_clinic", "Government clinic"),
        ("private_clinic", "Private clinic"),
        ("private_doctor", "Private doctor"),
        ("mocca_clinic", "MOCCA study clinic"),
        (UNKNOWN, "Don't recall"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.moccaoriginalsites": [
        ("amana", "Amana"),
        ("bunju", "Bunju"),
        ("hindu_mandal", "Hindu Mandal"),
        ("mkuranga", "Mkuranga"),
        ("mwananyamala", "Mwananyamala"),
        ("kisugu", "Kisugu"),
        ("kiswa", "Kiswa"),
        ("mulago", "Mulago"),
        ("ndejje", "Ndejje"),
        ("wakiso", "Wakiso"),
    ],
    "mocca_lists.conditions": [
        (HYPERTENSION, "Patient has high blood pressure (Hypertension)"),
        (DIABETES, "Patient has high blood sugar (Diabetes)"),
        (HIV, "Patient has HIV infection (HIV+)"),
    ],
    "mocca_lists.refillconditions": [
        (HYPERTENSION, "Hypertension"),
        (DIABETES, "Diabetes"),
        (HIV, "HIV"),
    ],
    "mocca_lists.reasonsfortesting": [
        ("patient_request", "Patient was well and made a request"),
        ("patient_complication", "Patient had a clinical complication"),
        ("signs_symptoms", "Patient had suggestive signs and symptoms"),
        (OTHER, "Other reason (specify below)"),
    ],
    "mocca_lists.offstudyreasons": [
        ("completed_followup", "Patient completed 12 months of follow-up"),
        (LOST_TO_FOLLOWUP, "Patient lost to follow-up"),
        (DEAD, "Patient reported/known to have died"),
        (WITHDRAWAL, "Patient withdrew consent to participate further"),
        (LATE_EXCLUSION, "Patient fulfilled late exclusion criteria*"),
        (TRANSFERRED, "Patient has been transferred to another health centre"),
        (
            OTHER,
            "Other reason (specify below)",
        ),
    ],
    "mocca_lists.htntreatments": [
        ("aldactone", "Aldactone (Spironolactone)"),
        ("amlodipine", "Amlodipine"),
        ("atenolol", "Atenolol"),
        ("atorvastatin", "Atorvastatin"),
        ("bendroflumethiazide", "Bendroflumethiazide"),
        ("bisoprolol", "Bisoprolol"),
        ("captopril", "Captopril"),
        ("carvedilol", "Carvedilol"),
        ("clopidogrel", "Clopidogrel"),
        ("enalapril", "Enalapril"),
        ("frusemide", "Frusemide"),
        ("hydralazine", "Hydralazine"),
        ("hydrochlorothiazide", "Hydrochlorothiazide"),
        ("junior_aspirin", "Junior Aspirin"),
        ("losartan_h", "losartan Hydrochlorothiazide (Losartan H/Repace H)"),
        ("losartan", "losartan"),
        ("metoprolol", "Metoprolol"),
        ("nifedipine", "Nifedipine"),
        ("propanolol", "Propanolol"),
        ("ramipril", "Ramipril"),
        ("simvastatin", "Simvastatin"),
        ("valsartan", "Valsartan"),
        (OTHER, "Other treatment (specify below)"),
    ],
    "mocca_lists.arvregimens": [
        ("ABC_3TC_ATV_r", "ABC + 3TC + ATV/r"),
        ("ABC_3TC_DTG", "ABC + 3TC + DTG"),
        ("ABC_3TC_EFV", "ABC + 3TC + EFV"),
        ("ABC_3TC_LPV_r", "ABC + 3TC + LPV/r"),
        ("AZT_3TC_ATV_r", "AZT + 3TC + ATV/r"),
        ("AZT_3TC_DTG", "AZT + 3TC + DTG"),
        ("AZT_3TC_EFV", "AZT + 3TC + EFV"),
        ("AZT_3TC_LPV_r", "AZT + 3TC + LPV/r"),
        ("AZT_3TC_NVP", "AZT + 3TC + NVP"),
        ("DTG_ABC/3TC_ATV_r", "DTG + (ABC/3TC) + ATV/r"),
        ("TDF_3TC_ATV_r", "TDF + 3TC + ATV/r"),
        ("TDF_3TC_DTG", "TDF + 3TC + DTG"),
        ("TDF_3TC_EFV", "TDF + 3TC + EFV"),
        ("TDF_3TC_LPV_r", "TDF + 3TC + LPV/r"),
        ("TDF_3TC_NVP", "TDF + 3TC + NVP"),
        ("TDF_FTC_ATV_r", "TDF + FTC + ATV/r"),
        ("TDF_FTC_DTG", "TDF + FTC + DTG"),
        ("TDF_FTC_EFV", "TDF + FTC + EFV"),
        ("TDF_FTC_LPV_r", "TDF + FTC + LPV/r"),
        ("TDF_FTC_NVP", "TDF + FTC + NVP"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.arvdrugs": [
        ("3TC", "3TC"),
        ("ABC", "ABC"),
        ("ATV_r", "ATV/r"),
        ("AZT", "AZT"),
        ("DTG", "DTG"),
        ("EFV", "EFV"),
        ("FTC", "FTC"),
        ("LPV_r", "LPV/r"),
        ("NVP", "NVP"),
        ("TDF", "TDF"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.visitreasons": [
        ("drug_refill", "Drug Refill"),
        ("clinic_review", "Clinic Review"),
        ("unwell", "Feeling unwell (self referral)"),
        ("unscheduled", "Unscheduled"),
    ],
    "mocca_lists.dmtreatments": [
        ("glibenclamide_metformin", "Glibenclamide + Metformin combo"),
        ("glibenclamide_s", "Glibenclamide (S)"),
        ("gliclazide_s", "Gliclazide (S)"),
        ("glimepiride_1mg_metformin", "Glimepiride (1mg) + Metformin combo"),
        ("glimepiride_2mg_metformin", "Glimepiride (2mg) + Metformin combo"),
        ("glimepiride_s", "Glimepiride (S)"),
        ("glipizide_s", "Glipizide (S)"),
        ("insulin", "Insulin"),
        ("metformin_b", "Metformin (B)"),
        ("pioglitazone", "Pioglitazone"),
        ("pregabalin", "Pregabalin (diabetic neuropathy)"),
        (
            "vitamin_b_folic_acid",
            "Vitamin Bs + Folic Acid (Neuroton- diabetic neuropathy)",
        ),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.choltreatments": [
        ("rosuvastatin", "Rosuvastatin"),
        ("simvastatin", "Simvastatin"),
        ("atorvastatin", "Atorvastatin"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.clinicservices": [
        (STUDY_DEFINED_TIMEPOINT, "Scheduled study visit"),
        (ROUTINE_VISIT, "Routine clinic review by a clinician"),
        (REFILL, "Drug refill"),
        (UNWELL_VISIT, "Feeling unwell"),
        ("club_or_health_activity", "Patient club / Health Activity"),
        (NOT_APPLICABLE, "Not applicable"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.rxmodifications": [
        ("dose_changes", "Dose changes"),
        ("drugs_substitution", "Drugs substitution"),
        ("drug_additions", "Additional drugs added to existing regimen"),
        ("some_stopped", "Some drugs stopped"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.rxmodificationreasons": [
        ("availability", "Limited availability of drugs"),
        ("side_effects", "Had side-effects"),
        ("feel_better", "Felt well and stopped/reduced drug prescription"),
        (OTHER, "Other, specify"),
    ],
    "mocca_lists.subjectvisitmissedreasons": [
        ("forgot", "Forgot / Can’t remember being told about appointment"),
        ("family_emergency", "Family emergency (e.g. funeral) and was away"),
        ("travelling", "Away travelling/visiting"),
        ("working_schooling", "Away working/schooling"),
        ("too_sick", "Too sick or weak to come to the centre"),
        ("lack_of_transport", "Transportation difficulty"),
        (HOSPITALIZED, "Hospitalized"),
        (OTHER, "Other reason (specify below)"),
    ],
    "mocca_lists.drugpaysources": [
        ("own_cash", "Own cash"),
        ("insurance", "Insurance"),
        ("club", "Patient support group / club"),
        ("relative", "Relative or others paying"),
        (FREE_OF_CHARGE, "Free drugs from the pharmacy"),
        (OTHER, "Other pay source (specify below)"),
    ],
    "mocca_lists.transportchoices": [
        ("bus", "Bus"),
        ("train", "Train"),
        ("ambulance", "Ambulance"),
        ("private_taxi", "Private taxi"),
        ("own_bicycle", "Own bicycle"),
        ("hired_motorbike", "Hired motorbike"),
        ("own_car", "Own car"),
        ("own_motorbike", "Own motorbike"),
        ("hired_bicycle", "Hired bicycle"),
        ("foot", "Foot"),
        (OTHER, "Other reason (specify below)"),
    ],
    "mocca_lists.nonadherencereasons": [
        ("forgot_to_take", "I simply forgot to take my medication"),
        ("travelled", "I travelled and forgot my medication"),
        ("feel_better", "I felt better and stopped taking my medication"),
        (
            "insufficient_supply",
            "I did not get enough medication from hospital/clinic, could not buy more",
        ),
        ("feel_ill", "The medications were making me feel sick"),
        ("too_many_pills", "Too many pills so I stopped / reduced"),
        (OTHER, "Other, please specify ..."),
    ],
}



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import NO, NOT_REQUIRED, YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator

from ..action_items import is_baseline
from ..models import HtnInitialReview, Indicators


class IndicatorsFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):

        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.required_if_true(
            is_baseline(self.cleaned_data.get("subject_visit")),
            field_required="weight",
            inverse=False,
        )
        self.required_if_true(
            is_baseline(self.cleaned_data.get("subject_visit")),
            field_required="height",
            inverse=False,
        )
        self.required_if(YES, field="r1_taken", field_required="sys_blood_pressure_r1")
        self.required_if(NO, field="r1_taken", field_required="r1_reason_not_taken")
        self.required_if(YES, field="r1_taken", field_required="dia_blood_pressure_r1")
        self.validate_bp_reading(
            "sys_blood_pressure_r1",
            "dia_blood_pressure_r1",
        )
        if self.cleaned_data.get("r2_taken") == NOT_REQUIRED and self.htn_initial_review:
            raise forms.ValidationError(
                {"r2_taken": "Invalid. Expected YES or NO. Patient is hypertensive."}
            )
        self.required_if(NO, field="r2_taken", field_required="r2_reason_not_taken")
        self.required_if(YES, field="r2_taken", field_required="sys_blood_pressure_r2")
        self.required_if(YES, field="r2_taken", field_required="dia_blood_pressure_r2")
        self.validate_bp_reading(
            "sys_blood_pressure_r2",
            "dia_blood_pressure_r2",
        )

    @property
    def htn_initial_review(self):
        try:
            return HtnInitialReview.objects.get(
                subject_visit__subject_identifier=self.cleaned_data.get(
                    "subject_visit"
                ).subject_identifier,
                report_datetime__lte=self.cleaned_data.get("report_datetime"),
            )
        except ObjectDoesNotExist:
            return None

    def validate_bp_reading(self, sys_field, dia_field):
        if self.cleaned_data.get(sys_field) and self.cleaned_data.get(dia_field):
            if self.cleaned_data.get(sys_field) < self.cleaned_data.get(dia_field):
                raise forms.ValidationError(
                    {dia_field: "Systolic must be greater than diastolic."}
                )


class IndicatorsForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = IndicatorsFormValidator

    class Meta:
        model = Indicators
        fields = "__all__"



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from .admin_site import mocca_lists_admin
from .models import (
    ArvDrugs,
    ArvRegimens,
    CholTreatments,
    ClinicServices,
    Conditions,
    DmTreatments,
    DrugPaySources,
    HtnTreatments,
    MoccaOriginalSites,
    NonAdherenceReasons,
    OffstudyReasons,
    ReasonsForTesting,
    RefillConditions,
    RxModificationReasons,
    RxModifications,
    SubjectVisitMissedReasons,
    TransportChoices,
    VisitReasons,
)


@admin.register(MoccaOriginalSites, site=mocca_lists_admin)
class MoccaOriginalSitesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(TransportChoices, site=mocca_lists_admin)
class TransportChoicesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(DrugPaySources, site=mocca_lists_admin)
class DrugPaySourcesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(SubjectVisitMissedReasons, site=mocca_lists_admin)
class SubjectVisitMissedReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(RefillConditions, site=mocca_lists_admin)
class RefillConditionsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(ReasonsForTesting, site=mocca_lists_admin)
class ReasonsForTestingAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(NonAdherenceReasons, site=mocca_lists_admin)
class NonAdherenceReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Conditions, site=mocca_lists_admin)
class ConditionsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(OffstudyReasons, site=mocca_lists_admin)
class OffstudyReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(HtnTreatments, site=mocca_lists_admin)
class HtnTreatmentsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(CholTreatments, site=mocca_lists_admin)
class CholTreatmentsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(ArvRegimens, site=mocca_lists_admin)
class ArvRegimensAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(VisitReasons, site=mocca_lists_admin)
class VisitReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(DmTreatments, site=mocca_lists_admin)
class DmTreatmentsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(ClinicServices, site=mocca_lists_admin)
class ClinicServicesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(ArvDrugs, site=mocca_lists_admin)
class ArvDrugsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(RxModifications, site=mocca_lists_admin)
class RxModificationsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(RxModificationReasons, site=mocca_lists_admin)
class RxModificationReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass



from edc_fieldsets import Fieldset

care_status_fieldset = Fieldset(
    "care",
    "care_not_in_reason",
    "care_facility_location",
    "icc",
    "icc_not_in_reason",
    "icc_since_mocca",
    "icc_since_mocca_comment",
    "care_comment",
    section="Care status",
).fieldset



# Generated by Django 3.2.11 on 2022-02-03 23:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_lists", "0005_auto_20210423_1451"),
        ("mocca_subject", "0031_alter_arvregimens_options"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="ArvRegimens",
            new_name="Rx",
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="rx",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="mocca_subject.rx",
                verbose_name="Which medicine did the patient receive today?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhiv",
            name="rx",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="mocca_subject.rx",
                verbose_name="Which medicine did the patient receive today?",
            ),
        ),
    ]



# Generated by Django 3.1.6 on 2021-03-02 04:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0017_auto_20210302_0748'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cholmedicationadherence',
            name='pill_count',
            field=models.IntegerField(null=True, verbose_name='Number of pills left in the bottle'),
        ),
        migrations.AlterField(
            model_name='dmmedicationadherence',
            name='pill_count',
            field=models.IntegerField(null=True, verbose_name='Number of pills left in the bottle'),
        ),
        migrations.AlterField(
            model_name='historicalcholmedicationadherence',
            name='pill_count',
            field=models.IntegerField(null=True, verbose_name='Number of pills left in the bottle'),
        ),
        migrations.AlterField(
            model_name='historicaldmmedicationadherence',
            name='pill_count',
            field=models.IntegerField(null=True, verbose_name='Number of pills left in the bottle'),
        ),
        migrations.AlterField(
            model_name='historicalhivmedicationadherence',
            name='last_missed_pill',
            field=models.CharField(choices=[('today', 'today'), ('yesterday', 'yesterday'), ('earlier_this_week', 'earlier this week'), ('last_week', 'last week'), ('lt_month_ago', 'less than a month ago'), ('gt_month_ago', 'more than a month ago'), ('NEVER', 'have never missed taking my pills')], max_length=25, verbose_name='When was the last time you missed taking your <U>condition_label</U> medication?'),
        ),
        migrations.AlterField(
            model_name='historicalhtnmedicationadherence',
            name='last_missed_pill',
            field=models.CharField(choices=[('today', 'today'), ('yesterday', 'yesterday'), ('earlier_this_week', 'earlier this week'), ('last_week', 'last week'), ('lt_month_ago', 'less than a month ago'), ('gt_month_ago', 'more than a month ago'), ('NEVER', 'have never missed taking my pills')], max_length=25, verbose_name='When was the last time you missed taking your <U>condition_label</U> medication?'),
        ),
        migrations.AlterField(
            model_name='hivmedicationadherence',
            name='last_missed_pill',
            field=models.CharField(choices=[('today', 'today'), ('yesterday', 'yesterday'), ('earlier_this_week', 'earlier this week'), ('last_week', 'last week'), ('lt_month_ago', 'less than a month ago'), ('gt_month_ago', 'more than a month ago'), ('NEVER', 'have never missed taking my pills')], max_length=25, verbose_name='When was the last time you missed taking your <U>condition_label</U> medication?'),
        ),
        migrations.AlterField(
            model_name='htnmedicationadherence',
            name='last_missed_pill',
            field=models.CharField(choices=[('today', 'today'), ('yesterday', 'yesterday'), ('earlier_this_week', 'earlier this week'), ('last_week', 'last week'), ('lt_month_ago', 'less than a month ago'), ('gt_month_ago', 'more than a month ago'), ('NEVER', 'have never missed taking my pills')], max_length=25, verbose_name='When was the last time you missed taking your <U>condition_label</U> medication?'),
        ),
    ]



from django import forms
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator

from ..models import DrugRefillHiv
from .drug_refill_mixins import DrugRefillFormValidatorMixin


class DrugRefillHivFormValidator(
    DrugRefillFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    def clean(self) -> None:
        self.drug_refill_clean()


class DrugRefillHivForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DrugRefillHivFormValidator

    class Meta:
        model = DrugRefillHiv
        fields = "__all__"



from django.db import models
from edc_constants.choices import YES_NO_NA, YES_NO_UNKNOWN
from edc_constants.constants import NOT_APPLICABLE
from edc_model_fields.fields import OtherCharField

from ..choices import CARE_SINCE_MOCCA, NOT_ICC_REASONS


class CareModelMixin(models.Model):
    care = models.CharField(
        verbose_name="Is the patient in care?",
        choices=YES_NO_UNKNOWN,
        max_length=25,
    )

    care_not_in_reason = models.CharField(
        verbose_name="If not in care, why?", max_length=25, null=True, blank=True
    )

    care_facility_location = models.CharField(
        verbose_name="Does the patient receive care in this facility",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="Either integrated care or vertical care",
    )

    icc_since_mocca = models.CharField(
        verbose_name=(
            "Has the patient received integrated care "
            "since the leaving the MOCCA (orig) trial until now?"
        ),
        max_length=25,
        choices=CARE_SINCE_MOCCA,
    )

    icc_since_mocca_comment = OtherCharField(
        verbose_name="If some interruption in integrated care, please explain"
    )

    icc = models.CharField(
        verbose_name="Does the patient currently receive integrated care",
        max_length=25,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text="Either at this facility or elsewhere",
    )

    icc_not_in_reason = models.CharField(
        verbose_name="If not receiving integrated care, why not?",
        max_length=25,
        choices=NOT_ICC_REASONS,
        default=NOT_APPLICABLE,
    )

    care_comment = models.TextField(
        verbose_name="Additional comments relevant to this patient's care",
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True



# Generated by Django 3.0.9 on 2020-11-18 21:32

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.firstname_field
import django_crypto_fields.fields.lastname_field
import django_revision.revision_field
import edc_model.models.fields.other_charfield
import edc_screening.model_mixins.screening_fields_model_mixin
import edc_sites.models
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models

import mocca_screening.mocca_original_sites


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("mocca_lists", "0001_initial"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.CreateModel(
            name="MoccaRegister",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "mocca_screening_identifier",
                    models.CharField(
                        blank=True,
                        help_text="If known",
                        max_length=15,
                        null=True,
                        verbose_name="MOCCA (original) screening identifier",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="MOCCA (original) study identifier",
                    ),
                ),
                (
                    "mocca_country",
                    models.CharField(
                        choices=[("uganda", "Uganda"), ("tanzania", "Tanzania")],
                        max_length=25,
                    ),
                ),
                (
                    "first_name",
                    django_crypto_fields.fields.firstname_field.FirstnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "last_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(18),
                            django.core.validators.MaxValueValidator(110),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ],
                    ),
                ),
                ("dob", models.DateField(blank=True, null=True)),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                        ],
                        default="unknown",
                        max_length=25,
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_lists.MoccaOriginalSites",
                    ),
                ),
            ],
            options={
                "verbose_name": "MOCCA Patient Register",
                "verbose_name_plural": "MOCCA Patient Register",
                "ordering": ["mocca_country", "mocca_site"],
            },
        ),
        migrations.CreateModel(
            name="SubjectScreening",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        db_index=True,
                        default="",
                        editable=False,
                        help_text="a field used for quick search",
                        max_length=250,
                        null=True,
                    ),
                ),
                (
                    "reference",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        unique=True,
                        verbose_name="Reference",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        blank=True,
                        editable=False,
                        max_length=50,
                        unique=True,
                        verbose_name="Screening ID",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="Date and time of report.",
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(110),
                        ]
                    ),
                ),
                (
                    "consent_ability",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Participant or legal guardian/representative able and willing to give informed consent.",
                    ),
                ),
                (
                    "unsuitable_for_study",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        help_text="If YES, patient NOT eligible, please give reason below.",
                        max_length=5,
                        verbose_name="Is there any other reason the patient is deemed to not be suitable for the study?",
                    ),
                ),
                (
                    "reasons_unsuitable",
                    models.TextField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not suitable for the study",
                    ),
                ),
                (
                    "unsuitable_agreed",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=5,
                        verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
                    ),
                ),
                ("eligible", models.BooleanField(default=False, editable=False)),
                (
                    "reasons_ineligible",
                    models.TextField(
                        editable=False,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not eligible",
                    ),
                ),
                (
                    "eligibility_datetime",
                    models.DateTimeField(
                        editable=False,
                        help_text="Date and time eligibility was determined",
                        null=True,
                    ),
                ),
                ("consented", models.BooleanField(default=False, editable=False)),
                ("refused", models.BooleanField(default=False, editable=False)),
                (
                    "screening_consent",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the subject given his/her verbal consent to be screened for the MOCCA Extension trial?",
                    ),
                ),
                (
                    "mocca_participant",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Was the patient enrolled to the original MOCCA study?",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="Original MOCCA study identifier",
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. Use `F`irst`L`ast or `L`ast`F`irst depending on the country custom.  (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ]
                    ),
                ),
                (
                    "mocca_register",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_lists.MoccaOriginalSites",
                        verbose_name="Original MOCCA site",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subject Screening",
                "verbose_name_plural": "Subject Screening",
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                (
                    "objects",
                    edc_screening.model_mixins.screening_fields_model_mixin.ScreeningManager(),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SubjectRefusal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("screening_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "reason",
                    models.CharField(
                        choices=[
                            ("unwilling_to_say", "I am unwilling to say"),
                            ("dont_have_time", "I don't have time"),
                            ("stigma", "I am worried about stigma"),
                            ("must_consult_spouse", "I need to consult my spouse"),
                            (
                                "dont_want_medication",
                                "I don't want to take any more medication",
                            ),
                            ("dont_want_to_join", "I don't want to take part"),
                            (
                                "need_to_think_about_it",
                                "I haven't had a chance to think about it",
                            ),
                            ("moving", "I am moving to another area"),
                            ("OTHER", "Other, please specify"),
                        ],
                        max_length=25,
                        verbose_name="Reason for refusal to join",
                    ),
                ),
                (
                    "other_reason",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subject Refusal",
                "verbose_name_plural": "Subject Refusals",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalSubjectScreening",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        db_index=True,
                        default="",
                        editable=False,
                        help_text="a field used for quick search",
                        max_length=250,
                        null=True,
                    ),
                ),
                (
                    "reference",
                    models.UUIDField(
                        db_index=True,
                        default=uuid.uuid4,
                        editable=False,
                        verbose_name="Reference",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        max_length=50,
                        verbose_name="Screening ID",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="Date and time of report.",
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(110),
                        ]
                    ),
                ),
                (
                    "consent_ability",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Participant or legal guardian/representative able and willing to give informed consent.",
                    ),
                ),
                (
                    "unsuitable_for_study",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        help_text="If YES, patient NOT eligible, please give reason below.",
                        max_length=5,
                        verbose_name="Is there any other reason the patient is deemed to not be suitable for the study?",
                    ),
                ),
                (
                    "reasons_unsuitable",
                    models.TextField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not suitable for the study",
                    ),
                ),
                (
                    "unsuitable_agreed",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=5,
                        verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
                    ),
                ),
                ("eligible", models.BooleanField(default=False, editable=False)),
                (
                    "reasons_ineligible",
                    models.TextField(
                        editable=False,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not eligible",
                    ),
                ),
                (
                    "eligibility_datetime",
                    models.DateTimeField(
                        editable=False,
                        help_text="Date and time eligibility was determined",
                        null=True,
                    ),
                ),
                ("consented", models.BooleanField(default=False, editable=False)),
                ("refused", models.BooleanField(default=False, editable=False)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "screening_consent",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the subject given his/her verbal consent to be screened for the MOCCA Extension trial?",
                    ),
                ),
                (
                    "mocca_participant",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Was the patient enrolled to the original MOCCA study?",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="Original MOCCA study identifier",
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. Use `F`irst`L`ast or `L`ast`F`irst depending on the country custom.  (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ]
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "mocca_register",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_lists.MoccaOriginalSites",
                        verbose_name="Original MOCCA site",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Subject Screening",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalSubjectRefusal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(db_index=True, max_length=50),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "reason",
                    models.CharField(
                        choices=[
                            ("unwilling_to_say", "I am unwilling to say"),
                            ("dont_have_time", "I don't have time"),
                            ("stigma", "I am worried about stigma"),
                            ("must_consult_spouse", "I need to consult my spouse"),
                            (
                                "dont_want_medication",
                                "I don't want to take any more medication",
                            ),
                            ("dont_want_to_join", "I don't want to take part"),
                            (
                                "need_to_think_about_it",
                                "I haven't had a chance to think about it",
                            ),
                            ("moving", "I am moving to another area"),
                            ("OTHER", "Other, please specify"),
                        ],
                        max_length=25,
                        verbose_name="Reason for refusal to join",
                    ),
                ),
                (
                    "other_reason",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Subject Refusal",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalDailyClosingLog",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "log_date",
                    models.DateField(
                        default=edc_utils.date.get_utcnow, verbose_name="Clinic date"
                    ),
                ),
                (
                    "clinic_services",
                    models.CharField(
                        choices=[
                            (
                                "integrated",
                                "Integrated care day (HIV, Diabetes, Hypertension)",
                            ),
                            ("ncd", "NCD day (Diabetes + Hypertension)"),
                            ("HIV", "HIV only day"),
                            ("diabetes", "Diabetes only day"),
                            ("hypertension", "Hypertension only day"),
                        ],
                        max_length=25,
                        verbose_name="Which services are being offered at the clinic today?",
                    ),
                ),
                (
                    "attended",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Total number of patients who attended the clinic today",
                    ),
                ),
                (
                    "selection_method",
                    models.CharField(
                        choices=[
                            ("random_sampling", "Random sampling"),
                            ("systematic", "Systematically selected"),
                            ("sequential", "Sequentially selected"),
                            ("purposively_selected", "Purposively selected"),
                        ],
                        max_length=25,
                        verbose_name="How were patients selected to be approached?",
                    ),
                ),
                (
                    "approached",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those who attended, how many were approached by the study team",
                    ),
                ),
                (
                    "agreed_to_screen",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those approached, how many agreed to be screened",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Daily Closing Log",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="DailyClosingLog",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "log_date",
                    models.DateField(
                        default=edc_utils.date.get_utcnow, verbose_name="Clinic date"
                    ),
                ),
                (
                    "clinic_services",
                    models.CharField(
                        choices=[
                            (
                                "integrated",
                                "Integrated care day (HIV, Diabetes, Hypertension)",
                            ),
                            ("ncd", "NCD day (Diabetes + Hypertension)"),
                            ("HIV", "HIV only day"),
                            ("diabetes", "Diabetes only day"),
                            ("hypertension", "Hypertension only day"),
                        ],
                        max_length=25,
                        verbose_name="Which services are being offered at the clinic today?",
                    ),
                ),
                (
                    "attended",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Total number of patients who attended the clinic today",
                    ),
                ),
                (
                    "selection_method",
                    models.CharField(
                        choices=[
                            ("random_sampling", "Random sampling"),
                            ("systematic", "Systematically selected"),
                            ("sequential", "Sequentially selected"),
                            ("purposively_selected", "Purposively selected"),
                        ],
                        max_length=25,
                        verbose_name="How were patients selected to be approached?",
                    ),
                ),
                (
                    "approached",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those who attended, how many were approached by the study team",
                    ),
                ),
                (
                    "agreed_to_screen",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those approached, how many agreed to be screened",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Daily Closing Log",
                "verbose_name_plural": "Daily Closing Logs",
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AddIndex(
            model_name="moccaregister",
            index=models.Index(
                fields=["mocca_country", "mocca_site"],
                name="mocca_scree_mocca_c_7f1635_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="moccaregister",
            index=models.Index(
                fields=["mocca_study_identifier", "initials", "gender"],
                name="mocca_scree_mocca_s_a92e02_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="moccaregister",
            constraint=models.UniqueConstraint(
                fields=("mocca_screening_identifier",),
                name="unique_mocca_screening_identifier",
            ),
        ),
        migrations.AddConstraint(
            model_name="moccaregister",
            constraint=models.UniqueConstraint(
                fields=("mocca_study_identifier",), name="unique_mocca_study_identifier"
            ),
        ),
        migrations.AddConstraint(
            model_name="moccaregister",
            constraint=models.UniqueConstraint(
                fields=("first_name", "last_name"), name="unique_first_name__last_name"
            ),
        ),
        migrations.AddConstraint(
            model_name="dailyclosinglog",
            constraint=models.UniqueConstraint(
                fields=("log_date", "site"), name="unique_date_for_site"
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import OtherBaselineDataForm
from ..models import OtherBaselineData
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(OtherBaselineData, site=mocca_subject_admin)
class OtherBaselineDataAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = OtherBaselineDataForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("Smoking", {"fields": ("smoking_status", "smoker_quit_ago")}),
        ("Alcohol", {"fields": ("alcohol", "alcohol_consumption")}),
        (
            "Other",
            {
                "fields": (
                    "employment_status",
                    "employment_status_other",
                    "education",
                    "marital_status",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "smoking_status": admin.VERTICAL,
        "alcohol": admin.VERTICAL,
        "alcohol_consumption": admin.VERTICAL,
        "employment_status": admin.VERTICAL,
        "education": admin.VERTICAL,
        "marital_status": admin.VERTICAL,
    }



# -*- coding: utf-8 -*-
import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.rst")) as readme:
    README = readme.read()

with open(os.path.join(os.path.dirname(__file__), "VERSION")) as f:
    VERSION = f.read().strip()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="mocca-edc",
    version=VERSION,
    author="Erik van Widenfelt",
    author_email="ew2789@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    url="https://github.com/mocca-trial/mocca-edc",
    license="GPL license, see LICENSE",
    description="MOCCA (ext) Trial EDC",
    long_description_content_type="text/x-rst",
    long_description=README,
    zip_safe=False,
    keywords="django MOCCA MOCCA RESPOND-AFRICA EDC",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    python_requires=">=3.7",
    test_suite="runtests.main",
)



# Generated by Django 3.1.6 on 2021-02-25 03:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('edc_action_item', '0028_auto_20210203_0706'),
        ('mocca_consent', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalsubjectconsent',
            options={'get_latest_by': 'history_date', 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical Subject Consent'},
        ),
        migrations.AlterModelOptions(
            name='subjectconsent',
            options={'get_latest_by': 'consent_datetime', 'ordering': ('created',), 'verbose_name': 'Subject Consent', 'verbose_name_plural': 'Subject Consents'},
        ),
        migrations.AlterField(
            model_name='historicalsubjectreconsent',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalsubjectreconsent',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='subjectreconsent',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='subjectreconsent',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='subjectreconsent',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='subjectreconsent',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='subjectreconsent',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
    ]



from django.db import models
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class NextAppointment(CrfModelMixin, edc_models.BaseUuidModel):

    appt_date = models.DateField(
        verbose_name="Next scheduled routine appointment",
        null=True,
        blank=True,
        help_text="if applicable.",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Routine Appointment"
        verbose_name_plural = "Routine Appointments"



from django.test import TestCase, tag
from edc_appointment.constants import INCOMPLETE_APPT
from edc_constants.constants import INCOMPLETE, NO, NOT_APPLICABLE, YES
from edc_dx import Diagnoses
from edc_utils import get_utcnow
from edc_visit_tracking.constants import UNSCHEDULED
from model_bakery import baker

from mocca_screening.constants import HIV_CLINIC
from mocca_subject.forms import MedicationsForm

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class TestMedications(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(
            report_datetime=get_utcnow(), clinic_type=HIV_CLINIC
        )
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening, clinic_type=HIV_CLINIC
        )

        self.subject_visit_baseline = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
        )
        baker.make(
            "mocca_subject.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_test=YES,
            hiv_dx=YES,
            hiv_test_ago="5y",
        )
        baker.make(
            "mocca_subject.hivinitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            arv_initiation_ago="4y",
        )

    @tag("med")
    def test_refill_applicable_if_dx(self):
        Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
            report_datetime=self.subject_visit_baseline.report_datetime,
            lte=True,
        )
        data = {
            "subject_visit": self.subject_visit_baseline,
            "report_datetime": self.subject_visit_baseline.report_datetime,
            "crf_status": INCOMPLETE,
            "refill_hiv": NOT_APPLICABLE,
            "refill_dm": NOT_APPLICABLE,
            "refill_htn": NOT_APPLICABLE,
        }
        form = MedicationsForm(data=data)
        form.is_valid()
        self.assertIn("refill_hiv", form._errors)
        data.update(refill_hiv=YES)
        form = MedicationsForm(data=data)
        form.is_valid()
        self.assertNotIn("refill_hiv", form._errors)

        self.subject_visit_baseline.appointment.appt_status = INCOMPLETE_APPT
        self.subject_visit_baseline.appointment.save()
        self.subject_visit_baseline.appointment.refresh_from_db()
        self.subject_visit_baseline.refresh_from_db()

        subject_visit = self.get_next_subject_visit(
            subject_visit=self.subject_visit_baseline, reason=UNSCHEDULED
        )

        baker.make(
            "mocca_subject.clinicalreview",
            subject_visit=subject_visit,
            hiv_test=NOT_APPLICABLE,
            hiv_dx=NOT_APPLICABLE,
            hiv_test_date=None,
        )

        data = {
            "subject_visit": subject_visit,
            "report_datetime": subject_visit.report_datetime,
            "crf_status": INCOMPLETE,
            "refill_hiv": NO,
            "refill_dm": NOT_APPLICABLE,
            "refill_htn": NOT_APPLICABLE,
        }

        form = MedicationsForm(data=data)
        form.is_valid()
        self.assertNotIn("refill_hiv", form._errors)

    @tag("med")
    def test_metadata_requires2(self):
        baker.make(
            "mocca_subject.medications",
            subject_visit=self.subject_visit_baseline,
        )
